
import os
import sys
import math
import numpy as np
from bisect import bisect_left
import matplotlib.pyplot as plt
from scipy import interpolate

def calculatePhs(sigma, wavelength, inclination, thickness, threshold, ranges, stdevkev=50):
    EOfrag = [1420, 780, 1750, 960] # KeV, energy max fragments Alpha 94, Li 94, aplha 6, Li 6
    if threshold < 2:
        threshold = 2
        print("Threshold set to 2")
    EOfrag = [1420, 780, 1750, 960]

    #pf = '/Users/francescopiscitelli/Documents/MATLAB/000_data/SRIM_real_10B4C_layer_Linkoping';
    #fprintf('\n Linkoping coatings used as default for stopping power! \n')

    Efrag = np.loadtxt(fname=os.path.dirname(os.path.abspath(__file__)) + "/data/B10/10B4C224/Erim_fragments.txt", unpack=True)

    L = np.arange(0.001, 8, 0.05) #distance traveled in boron

    limitRange = ranges
    p=[[],[]]
    energ=[[],[],[],[]]
    defrag=[[],[],[],[]]

    dl = np.diff(Efrag[4])
    for d in range(0, len(defrag)):
        defrag[d] = np.abs(np.diff(Efrag[d])/dl)
    #Defrag = [Defrag ; Defrag(end, 1: 4)];
    pe = [[],[],[],[]]
    peT = [[],[],[],[]]
    for k in range(0,len(L)):
        ptemp = PHSprob(L[k],thickness,sigma[0])
        p[0].append(ptemp[0]) #BS
        p[1].append(ptemp[1]) #T
        for j in range (0,4):
            if L[k] > 0.05 and L[k] < ranges[j]: # um(SRIM lower limit)
                index = find_nearest(Efrag[4],L[k])
                energ[j].append(Efrag[j][index])
                pe[j].append(p[0][k]/defrag[j][index])
                peT[j].append(p[1][k] / defrag[j][index])
            elif L[k] <= 0.05:
                energ[j].append(EOfrag[j])
                pe[j].append(p[0][k]/defrag[j][101])
                peT[j].append(p[1][k]/defrag[j][101])
            else:
                energ[j].append(0)
                pe[j].append(0)
                peT[j].append(0)
    prob = [np.array(pe[0])*94/100,np.array(pe[1])*94/100,np.array(pe[2])*6/100,np.array(pe[3])*6/100]
    probT = [np.array(peT[0])*94/100,np.array(peT[1])*94/100,np.array(peT[2])*6/100,np.array(peT[3])*6/100]

    #prob = [0 0 0 0;prob];
    #probT = [0 0 0 0;probT];
    energ = [EOfrag,energ]


   # DE = np.diff(energ,1,1)
    #DE = [0 0 0 0;DE];
  #  effic = np.diag(prob*DE)



    return energ, pe, peT

def find_nearest(array,value):
    idx = (np.abs(array-value)).argmin()
    return idx


def PHSprob(l, thickness, sigma):
    p=[[],[]]
    if l >= 0 and l <= thickness:
        #BS
        p[0] = (1./2.)*(1./(l**2.))*( (1./sigma) - (( l+(1./sigma) )*np.exp([-l*sigma])) )[0]
        #TS
        p[1] = (1/2)*(1/(l**2))*(np.exp(-thickness*sigma))*( ((np.exp(l*sigma))*l) - (1/sigma)*((np.exp(l*sigma))-1) )
    elif l>thickness:
        #BS
        p[0] = (1 / 2) * (1 / (l ** 2)) * ((1 / sigma) - ((thickness + (1 / sigma)) * np.exp(-thickness * sigma)))
        #Transmission
        p[1] = (1 / 2) * (1 / (l ** 2)) * (thickness - (1 / sigma) * (1 - np.exp(-thickness * sigma)))
    else:
        p[0]=0
        p[1]=0

    return p



import os
import numpy as np
import math
import matplotlib.pyplot as plt
from scipy import interpolate


def aluminium(substrate, wavelength, inclination):
    """calculates delta of aluminium

    Args:
        substrate (int):  thickness of aluminium layer in um
        wavelength (array[1]):  array of
    Returns:
        thValue: point where y in the integral have the closest value to threshold



    ..  Original source in Matlab: https: // bitbucket.org / europeanspallationsource / dg_matlabborontools / src / bcbac538ad10d074c5150a228847efc2e0269e0d / B10tools / rangesMAT.m?at = default & fileviewer = file - view - default

    """
    #g/cm3
    densityAl = 2.7
    denAl = (densityAl * 6.022e23 / 26.98) * 1e-4
    temp = read_cross_section(wavelength)
    sigmaal=[]
    for t in temp:
        sigmaal.append(denAl * t * 1e-24)
    delta = []
    for s in sigmaal:
        delta.append(np.exp(-1*(s * substrate) / math.sin(math.radians(inclination))))
    return delta


def read_cross_section(lambdalist):
    ht = 1.054e-34
    nmass = 1.67e-27
    # lambdaList transformed to ev
    lamen = []
    sigma = []
    c0 = 0
    for l in lambdalist:
        lamen.append(((ht ** 2) * 4 * math.pi ** 2) / (2 * nmass * (l[0] ** 2)) * 1e20 * 6.24e18)

    x, y = np.loadtxt(fname=os.path.dirname(os.path.abspath(__file__)) + "/data/Aluminium/AlCrossSect_(n,g).py",
                      delimiter=',', unpack=True)
    xlog = []
    ylog = []
    lamenlog = []
    for v in x:
        xlog.append(math.log10(v))
    for v in y:
        ylog.append(math.log10(v))
    for v in lamen:
        lamenlog.append(math.log10(v))
    f = interpolate.interp1d(xlog, ylog, kind='cubic')
    for v in lamenlog:
        sigma.append(10 ** f(v))
    # check if interpolation works
    # xnew = np.linspace(xlog[0], xlog[100], num=41, endpoint=True)
    # plt.plot(xlog, ylog, 'o', xnew, f(xnew), '--')
    print (lamen)
    print (sigma)
    plt.show()
    return sigma

